/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2016                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/export_off.h>

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public vcg::UsedTypes<vcg::Use<MyVertex>   ::AsVertexType,
                                           vcg::Use<MyEdge>     ::AsEdgeType,
                                           vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<   MyUsedTypes, vcg::face::FFAdj,  vcg::face::VertexRef, vcg::face::Color4b, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<   MyUsedTypes> {};

class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};

using namespace vcg;    

int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.off>\n");
    return -1;
  }
  /*!
    */
  MyMesh m;

  if(tri::io::ImporterOFF<MyMesh>::Open(m,argv[1])!=tri::io::ImporterOFF<MyMesh>::NoError)
  {
    printf("Error reading file  %s\n",argv[1]);
    exit(0);
  }

  tri::RequirePerVertexNormal(m);
  tri::UpdateTopology<MyMesh>::FaceFace(m);
  
  int b_en = 0;
 
  for(auto fi=m.face.begin();fi!=m.face.end();++fi)
  {
    if(!(*fi).IsD())  // this could be useless if the mesh has no deleted faces (i.e. if it has been compacted)
      for(int i=0;i<3;++i)
        if((*fi).FFp(i)==&(*fi))
          ++b_en;
  }

  int en = (m.FN()*3 - b_en)/2 + b_en; 

  printf("Input mesh  vn:%i en: %i fn:%i, chi= %i\n",m.VN(), en, m.FN(), m.VN()-en+m.FN());
  
  // unmark all the vertices 
  tri::UpdateFlags<MyMesh>::VertexClearV(m);
int border_loop_count = 0;
  // search for a boundary face 
for(auto fi=m.face.begin();fi!=m.face.end();++fi)
  {
    for(int i=0;i<3;++i)
    {
      if(face::IsBorder(*fi,i) && !(*fi).V(i)->IsV())
      {// mark visited the vertices of the border edge
        (*fi).V (i)->SetV();
        (*fi).V1(i)->SetV(); // you can use V1(i) instead of V((i+1)%3)
        // now start a walk along the border
        ++border_loop_count;
        face::Pos<MyFace> start(&*fi,i);
        face::Pos<MyFace> cur = start;
        do
        {        
          assert(cur.IsBorder());
          do
          {
            cur.FlipE();
            cur.FlipF();
          } while(!cur.IsBorder());

          cur.FlipV();  
          cur.V()->SetV();
          cur.F()->C()=Color4b::Red;
        } while (cur!=start);
      }

    }  
  }
  printf("Found %i border loops\n",border_loop_count);
  
  // save the mesh with the border faces colored in red
  tri::io::ExporterOFF<MyMesh>::Save(m,"border.off",tri::io::Mask::IOM_FACECOLOR);
  return 0;
}
